@@ -1,3 +1,28 @@
+/*
+ * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
+#ifndef _MACH_O_REDO_PREBINDING_H_
+#define _MACH_O_REDO_PREBINDING_H_
+
 #define REDO_PREBINDING_VERSION 2
 #include <mach/machine.h>
 /*
@@ -23,6 +48,10 @@
  * dependent library paths. 
  */
 
+#if __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /*
  * dependent_libs() takes a file_name of a binary and returns a malloc(3)'ed
  * array of pointers (NULL terminated) to names (also malloc(3)'ed and '\0'
@@ -38,6 +67,21 @@ const char *file_name,
 const char *program_name,
 char **error_message);
 
+/*
+ * install_name() takes a file_name of a binary and returns a malloc(3)'ed
+ * pointer to a NULL terminated string containing the install_name value for
+ * the binary. If unsuccessful install_name() returns NULL.  In particular,
+ * NULL is returned if the binary is not a dylib and there is no error_message
+ * set.  If the all of the arch's are dylibs but all the install names don't
+ * match NULL is returned and a error_message is set.  If some but not all of
+ * the archs are dylibs NULL is returned and a error_message is set.
+ */ 
+extern
+char *
+install_name(
+const char *file_name,
+const char *program_name,
+char **error_message);
 
 /* return values for redo_prebinding() */
 enum redo_prebinding_retval {
@@ -133,6 +177,53 @@ unsigned long expected_address,
 cpu_type_t allow_missing_architectures);
 
 
+/*
+ * unprebind() takes a file_name of a binary and resets or removes prebinding
+ * information from it.  If inbuf is non-NULL, the memory pointed to by inbuf is
+ * used as the input file contents.  Otherwise, the contents are loaded from 
+ * the file at path file_name.  Even if inbuf is non-NULL, a file_name 
+ * parameter should be specified for error reporting.  Similarly, if outbuf is 
+ * non-NULL, upon return, outbuf will point to a buffer containing the 
+ * unprebound binary and outlen will point to the length of the output buffer.  
+ * This buffer is vm_allocate'd and therefore should be vm_deallocate'd when it 
+ * is no longer needed.  If outbuf is NULL, and output_file is not NULL the 
+ * update file is written to output_file, if outbuf is NULL and output_file is 
+ * NULL, it is written to file_name.  
+ * If unprebind() is successful it returns REDO_PREBINDING_SUCCESS otherwise it
+ * returns REDO_PREBINDING_FAILURE If the binary is already unprebound (i.e. it
+ * has the MH_PREBINDABLE flag set) then REDO_PREBINDING_NOT_NEEDED is returned.
+ * If the binary is not prebound and not prebindable, 
+ * REDO_PREBINDING_NOT_PREBOUND is returned.  If zero_checksum is non-zero then
+ * the cksum field the LC_PREBIND_CKSUM load command (if any) is set to zero on
+ * output, otherwise it is left alone.
+ * Unprebinding slides dynamic libraries to address zero, resets prebound 
+ * symbols to address zero and type undefined, resets symbol pointers, removes 
+ * LC_PREBOUND_DYLIB commands, resets library timestamps, resets two-level hints
+ * and updates relocation entries if necessary.  Unprebound binaries have
+ * the MH_PREBINDABLE flag set, but not MH_PREBOUND.  It will also set the the
+ * MH_ALLMODSBOUND flag if all two-level libraries were used and all modules
+ * were found to be bound in the LC_PREBOUND_DYLIB commands.
+ * As unprebinding is intended to produce a canonical Mach-O
+ * binary, bundles and non-prebound executables and dylibs are acceptable
+ * as input.  For these files, the  unprebind operation will zero library 
+ * time stamps and version numbers and zero entries in the two-level hints
+ * table.  These files will not gain the MH_PREBINDABLE flag.
+ * All resulting binaries successfully processed by unprebind() will have
+ * the MH_CANONICAL flag.
+ */
+extern
+enum redo_prebinding_retval
+unprebind(
+const char *file_name,
+const char *output_file,
+const char *program_name,
+char **error_message,
+int zero_checksum,
+void *inbuf,
+unsigned long inlen,
+void **outbuf,
+unsigned long *outlen);
+
 enum object_file_type_retval {
     OFT_OTHER,
     OFT_EXECUTABLE,
@@ -178,3 +269,9 @@ struct prebind_cksum_arch **cksums,
 unsigned long *ncksums,
 const char *program_name,
 char **error_message);
+
+#if __cplusplus
+};
+#endif /* __cplusplus */
+
+#endif /* _MACH_O_REDO_PREBINDING_H_ */
