@@ -20,13 +20,21 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+#include <mach/mach.h>
+
 /* The currently known Mac OS X deployment targets */
 enum macosx_deployment_target_value {
     MACOSX_DEPLOYMENT_TARGET_10_1,
     MACOSX_DEPLOYMENT_TARGET_10_2,
-    MACOSX_DEPLOYMENT_TARGET_10_3
+    MACOSX_DEPLOYMENT_TARGET_10_3,
+    MACOSX_DEPLOYMENT_TARGET_10_4,
+    MACOSX_DEPLOYMENT_TARGET_10_5
 };
 
 __private_extern__ void get_macosx_deployment_target(
     enum macosx_deployment_target_value *value,
-    const char **name);
+    const char **name,
+    cpu_type_t cputype);
+
+__private_extern__ void put_macosx_deployment_target(
+    char *target);
