@@ -103,6 +103,34 @@
 #define __xmm_reg xmm_reg
 #define __zdiv zdiv
 
+#define __rax rax
+#define __rbx rbx
+#define __rcx rcx
+#define __rdx rdx
+#define __rdi rdi
+#define __rsi rsi
+#define __rbp rbp
+#define __rsp rsp
+#define __r8 r8
+#define __r9 r9
+#define __r10 r10
+#define __r11 r11
+#define __r12 r12
+#define __r13 r13
+#define __r14 r14
+#define __r15 r15
+#define __rip rip
+#define __rflags rflags
+
+#define __dr0 dr0
+#define __dr1 dr1
+#define __dr2 dr2
+#define __dr3 dr3
+#define __dr4 dr4
+#define __dr5 dr5
+#define __dr6 dr6
+#define __dr7 dr7
+
 #include <string.h>
 #import <mach-o/i386/swap.h>
 
@@ -129,6 +157,298 @@ enum NXByteOrder target_byte_sex)
 	cpu->gs = NXSwapLong(cpu->gs);
 }
 
+#ifdef x86_THREAD_STATE64
+void
+swap_x86_thread_state64(
+x86_thread_state64_t *cpu,
+enum NXByteOrder target_byte_sex)
+{
+	cpu->rax = NXSwapLongLong(cpu->rax);
+	cpu->rbx = NXSwapLongLong(cpu->rbx);
+	cpu->rcx = NXSwapLongLong(cpu->rcx);
+	cpu->rdx = NXSwapLongLong(cpu->rdx);
+	cpu->rdi = NXSwapLongLong(cpu->rdi);
+	cpu->rsi = NXSwapLongLong(cpu->rsi);
+	cpu->rbp = NXSwapLongLong(cpu->rbp);
+	cpu->rsp = NXSwapLongLong(cpu->rsp);
+	cpu->rflags = NXSwapLongLong(cpu->rflags);
+	cpu->rip = NXSwapLongLong(cpu->rip);
+	cpu->r8 = NXSwapLongLong(cpu->r8);
+	cpu->r9 = NXSwapLongLong(cpu->r9);
+	cpu->r10 = NXSwapLongLong(cpu->r10);
+	cpu->r11 = NXSwapLongLong(cpu->r11);
+	cpu->r12 = NXSwapLongLong(cpu->r12);
+	cpu->r13 = NXSwapLongLong(cpu->r13);
+	cpu->r14 = NXSwapLongLong(cpu->r14);
+	cpu->r15 = NXSwapLongLong(cpu->r15);
+	cpu->cs = NXSwapLongLong(cpu->cs);
+	cpu->fs = NXSwapLongLong(cpu->fs);
+	cpu->gs = NXSwapLongLong(cpu->gs);
+}
+
+void
+swap_x86_state_hdr(
+x86_state_hdr_t *hdr,
+enum NXByteOrder target_byte_sex)
+{
+	hdr->flavor = NXSwapLong(hdr->flavor);
+	hdr->count = NXSwapLong(hdr->count);
+}
+
+void
+swap_x86_float_state64(
+x86_float_state64_t *fpu,
+enum NXByteOrder target_byte_sex)
+{
+    struct swapped_fp_control {
+	union {
+	    struct {
+		unsigned short
+			    :3,
+		    /*inf*/ :1,
+		    rc	    :2,
+		    pc	    :2,
+			    :2,
+		    precis  :1,
+		    undfl   :1,
+		    ovrfl   :1,
+		    zdiv    :1,
+		    denorm  :1,
+		    invalid :1;
+	    } fields;
+	    unsigned short half;
+	} u;
+    } sfpc;
+
+    struct swapped_fp_status {
+	union {
+	    struct {
+		unsigned short
+		    busy    :1,
+		    c3	    :1,
+		    tos	    :3,
+		    c2	    :1,
+		    c1	    :1,
+		    c0	    :1,
+		    errsumm :1,
+		    stkflt  :1,
+		    precis  :1,
+		    undfl   :1,
+		    ovrfl   :1,
+		    zdiv    :1,
+		    denorm  :1,
+		    invalid :1;
+	    } fields;
+	    unsigned short half;
+	} u;
+    } sfps;
+
+    enum NXByteOrder host_byte_sex;
+
+	host_byte_sex = NXHostByteOrder();
+
+	fpu->fpu_reserved[0] = NXSwapLong(fpu->fpu_reserved[0]);
+	fpu->fpu_reserved[1] = NXSwapLong(fpu->fpu_reserved[1]);
+
+	if(target_byte_sex == host_byte_sex){
+	    memcpy(&sfpc, &(fpu->fpu_fcw),
+		   sizeof(struct swapped_fp_control));
+	    sfpc.u.half = NXSwapShort(sfpc.u.half);
+	    fpu->fpu_fcw.rc = sfpc.u.fields.rc;
+	    fpu->fpu_fcw.pc = sfpc.u.fields.pc;
+	    fpu->fpu_fcw.precis = sfpc.u.fields.precis;
+	    fpu->fpu_fcw.undfl = sfpc.u.fields.undfl;
+	    fpu->fpu_fcw.ovrfl = sfpc.u.fields.ovrfl;
+	    fpu->fpu_fcw.zdiv = sfpc.u.fields.zdiv;
+	    fpu->fpu_fcw.denorm = sfpc.u.fields.denorm;
+	    fpu->fpu_fcw.invalid = sfpc.u.fields.invalid;
+
+	    memcpy(&sfps, &(fpu->fpu_fsw),
+		   sizeof(struct swapped_fp_status));
+	    sfps.u.half = NXSwapShort(sfps.u.half);
+	    fpu->fpu_fsw.busy = sfps.u.fields.busy;
+	    fpu->fpu_fsw.c3 = sfps.u.fields.c3;
+	    fpu->fpu_fsw.tos = sfps.u.fields.tos;
+	    fpu->fpu_fsw.c2 = sfps.u.fields.c2;
+	    fpu->fpu_fsw.c1 = sfps.u.fields.c1;
+	    fpu->fpu_fsw.c0 = sfps.u.fields.c0;
+	    fpu->fpu_fsw.errsumm = sfps.u.fields.errsumm;
+	    fpu->fpu_fsw.stkflt = sfps.u.fields.stkflt;
+	    fpu->fpu_fsw.precis = sfps.u.fields.precis;
+	    fpu->fpu_fsw.undfl = sfps.u.fields.undfl;
+	    fpu->fpu_fsw.ovrfl = sfps.u.fields.ovrfl;
+	    fpu->fpu_fsw.zdiv = sfps.u.fields.zdiv;
+	    fpu->fpu_fsw.denorm = sfps.u.fields.denorm;
+	    fpu->fpu_fsw.invalid = sfps.u.fields.invalid;
+	}
+	else{
+	    sfpc.u.fields.rc = fpu->fpu_fcw.rc;
+	    sfpc.u.fields.pc = fpu->fpu_fcw.pc;
+	    sfpc.u.fields.precis = fpu->fpu_fcw.precis;
+	    sfpc.u.fields.undfl = fpu->fpu_fcw.undfl;
+	    sfpc.u.fields.ovrfl = fpu->fpu_fcw.ovrfl;
+	    sfpc.u.fields.zdiv = fpu->fpu_fcw.zdiv;
+	    sfpc.u.fields.denorm = fpu->fpu_fcw.denorm;
+	    sfpc.u.fields.invalid = fpu->fpu_fcw.invalid;
+	    sfpc.u.half = NXSwapShort(sfpc.u.half);
+	    memcpy(&(fpu->fpu_fcw), &sfpc,
+		   sizeof(struct swapped_fp_control));
+
+	    sfps.u.fields.busy = fpu->fpu_fsw.busy;
+	    sfps.u.fields.c3 = fpu->fpu_fsw.c3;
+	    sfps.u.fields.tos = fpu->fpu_fsw.tos;
+	    sfps.u.fields.c2 = fpu->fpu_fsw.c2;
+	    sfps.u.fields.c1 = fpu->fpu_fsw.c1;
+	    sfps.u.fields.c0 = fpu->fpu_fsw.c0;
+	    sfps.u.fields.errsumm = fpu->fpu_fsw.errsumm;
+	    sfps.u.fields.stkflt = fpu->fpu_fsw.stkflt;
+	    sfps.u.fields.precis = fpu->fpu_fsw.precis;
+	    sfps.u.fields.undfl = fpu->fpu_fsw.undfl;
+	    sfps.u.fields.ovrfl = fpu->fpu_fsw.ovrfl;
+	    sfps.u.fields.zdiv = fpu->fpu_fsw.zdiv;
+	    sfps.u.fields.denorm = fpu->fpu_fsw.denorm;
+	    sfps.u.fields.invalid = fpu->fpu_fsw.invalid;
+	    sfps.u.half = NXSwapShort(sfps.u.half);
+	    memcpy(&(fpu->fpu_fsw), &sfps,
+		   sizeof(struct swapped_fp_status));
+	}
+	fpu->fpu_fop = NXSwapShort(fpu->fpu_fop);
+	fpu->fpu_ip = NXSwapLong(fpu->fpu_ip);
+	fpu->fpu_cs = NXSwapShort(fpu->fpu_cs);
+	fpu->fpu_rsrv2 = NXSwapShort(fpu->fpu_rsrv2);
+	fpu->fpu_dp = NXSwapLong(fpu->fpu_dp);
+	fpu->fpu_ds = NXSwapShort(fpu->fpu_ds);
+	fpu->fpu_rsrv3 = NXSwapShort(fpu->fpu_rsrv3);
+	fpu->fpu_mxcsr = NXSwapLong(fpu->fpu_mxcsr);
+	fpu->fpu_mxcsrmask = NXSwapLong(fpu->fpu_mxcsrmask);
+	fpu->fpu_reserved1 = NXSwapLong(fpu->fpu_reserved1);
+}
+
+void
+swap_x86_exception_state64(
+x86_exception_state64_t *exc,
+enum NXByteOrder target_byte_sex)
+{
+	exc->trapno = NXSwapLong(exc->trapno);
+	exc->err = NXSwapLong(exc->err);
+    	exc->faultvaddr = NXSwapLongLong(exc->faultvaddr);
+}
+
+void
+swap_x86_thread_state(
+x86_thread_state_t *cpu,
+enum NXByteOrder target_byte_sex)
+{
+    x86_state_hdr_t hdr;
+    enum NXByteOrder host_byte_sex;
+
+	host_byte_sex = NXHostByteOrder();
+	hdr = cpu->tsh;
+	if(target_byte_sex == host_byte_sex)
+	    swap_x86_state_hdr(&hdr, host_byte_sex);
+
+	swap_x86_state_hdr(&cpu->tsh, target_byte_sex);
+
+	if(hdr.flavor == x86_THREAD_STATE32)
+	    swap_i386_thread_state(&cpu->uts.ts32, target_byte_sex);
+	else if(hdr.flavor == x86_THREAD_STATE64)
+	    swap_x86_thread_state64(&cpu->uts.ts64, target_byte_sex);
+}
+
+void
+swap_x86_float_state(
+x86_float_state_t *fpu,
+enum NXByteOrder target_byte_sex)
+{
+    x86_state_hdr_t hdr;
+    enum NXByteOrder host_byte_sex;
+
+	host_byte_sex = NXHostByteOrder();
+	hdr = fpu->fsh;
+	if(target_byte_sex == host_byte_sex)
+	    swap_x86_state_hdr(&hdr, host_byte_sex);
+
+	swap_x86_state_hdr(&fpu->fsh, target_byte_sex);
+
+	if(hdr.flavor == x86_FLOAT_STATE32)
+	    swap_i386_float_state(&fpu->ufs.fs32, target_byte_sex);
+	else if(hdr.flavor == x86_FLOAT_STATE64)
+	    swap_x86_float_state64(&fpu->ufs.fs64, target_byte_sex);
+}
+
+void
+swap_x86_exception_state(
+x86_exception_state_t *exc,
+enum NXByteOrder target_byte_sex)
+{
+    x86_state_hdr_t hdr;
+    enum NXByteOrder host_byte_sex;
+
+	host_byte_sex = NXHostByteOrder();
+	hdr = exc->esh;
+	if(target_byte_sex == host_byte_sex)
+	    swap_x86_state_hdr(&hdr, host_byte_sex);
+
+	swap_x86_state_hdr(&exc->esh, target_byte_sex);
+
+	if(hdr.flavor == x86_EXCEPTION_STATE32)
+	    swap_i386_exception_state(&exc->ues.es32, target_byte_sex);
+	else if(hdr.flavor == x86_EXCEPTION_STATE64)
+	    swap_x86_exception_state64(&exc->ues.es64, target_byte_sex);
+}
+
+void
+swap_x86_debug_state32(
+x86_debug_state32_t *debug,
+enum NXByteOrder target_byte_sex)
+{
+	debug->dr0 = NXSwapLong(debug->dr0);
+	debug->dr1 = NXSwapLong(debug->dr1);
+	debug->dr2 = NXSwapLong(debug->dr2);
+	debug->dr3 = NXSwapLong(debug->dr3);
+	debug->dr4 = NXSwapLong(debug->dr4);
+	debug->dr5 = NXSwapLong(debug->dr5);
+	debug->dr6 = NXSwapLong(debug->dr6);
+	debug->dr7 = NXSwapLong(debug->dr7);
+}
+
+void
+swap_x86_debug_state64(
+x86_debug_state64_t *debug,
+enum NXByteOrder target_byte_sex)
+{
+	debug->dr0 = NXSwapLongLong(debug->dr0);
+	debug->dr1 = NXSwapLongLong(debug->dr1);
+	debug->dr2 = NXSwapLongLong(debug->dr2);
+	debug->dr3 = NXSwapLongLong(debug->dr3);
+	debug->dr4 = NXSwapLongLong(debug->dr4);
+	debug->dr5 = NXSwapLongLong(debug->dr5);
+	debug->dr6 = NXSwapLongLong(debug->dr6);
+	debug->dr7 = NXSwapLongLong(debug->dr7);
+}
+
+void
+swap_x86_debug_state(
+x86_debug_state_t *debug,
+enum NXByteOrder target_byte_sex)
+{
+    x86_state_hdr_t hdr;
+    enum NXByteOrder host_byte_sex;
+
+	host_byte_sex = NXHostByteOrder();
+	hdr = debug->dsh;
+	if(target_byte_sex == host_byte_sex)
+	    swap_x86_state_hdr(&hdr, host_byte_sex);
+
+	swap_x86_state_hdr(&debug->dsh, target_byte_sex);
+
+	if(hdr.flavor == x86_DEBUG_STATE32)
+	    swap_x86_debug_state32(&debug->uds.ds32, target_byte_sex);
+	else if(hdr.flavor == x86_DEBUG_STATE64)
+	    swap_x86_debug_state64(&debug->uds.ds64, target_byte_sex);
+}
+#endif /* x86_THREAD_STATE64 */
+
 /* current i386 thread states */
 #if i386_THREAD_STATE == 1
 void
@@ -268,7 +588,7 @@ enum NXByteOrder target_byte_sex)
 void
 swap_i386_exception_state(
 i386_exception_state_t *exc,
-enum NXByteOrder target_byte_order)
+enum NXByteOrder target_byte_sex)
 {
 	exc->trapno = NXSwapLong(exc->trapno);
 	exc->err = NXSwapLong(exc->err);
