@@ -20,8 +20,10 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
-#import <mach-o/swap.h>
-#import <string.h>
+#ifndef RLD
+#include <stdint.h>
+#include <mach-o/swap.h>
+#include <string.h>
 
 void
 swap_fat_header(
@@ -35,10 +37,10 @@ enum NXByteOrder target_byte_sex)
 void
 swap_fat_arch(
 struct fat_arch *fat_archs,
-unsigned long nfat_arch,
+uint32_t nfat_arch,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
 
 	for(i = 0; i < nfat_arch; i++){
 	    fat_archs[i].cputype    = NXSwapLong(fat_archs[i].cputype);
@@ -63,6 +65,21 @@ enum NXByteOrder target_byte_sex)
 	mh->flags = NXSwapLong(mh->flags);
 }
 
+void
+swap_mach_header_64(
+struct mach_header_64 *mh,
+enum NXByteOrder target_byte_sex)
+{
+	mh->magic = NXSwapLong(mh->magic);
+	mh->cputype = NXSwapLong(mh->cputype);
+	mh->cpusubtype = NXSwapLong(mh->cpusubtype);
+	mh->filetype = NXSwapLong(mh->filetype);
+	mh->ncmds = NXSwapLong(mh->ncmds);
+	mh->sizeofcmds = NXSwapLong(mh->sizeofcmds);
+	mh->flags = NXSwapLong(mh->flags);
+	mh->reserved = NXSwapLong(mh->reserved);
+}
+
 void
 swap_load_command(
 struct load_command *lc,
@@ -90,13 +107,31 @@ enum NXByteOrder target_byte_sex)
 	sg->flags = NXSwapLong(sg->flags);
 }
 
+void 
+swap_segment_command_64(
+struct segment_command_64* sg,
+enum NXByteOrder target_byte_sex)
+{
+	/* char segname[16] */
+	sg->cmd = NXSwapLong(sg->cmd);
+	sg->cmdsize = NXSwapLong(sg->cmdsize);
+	sg->vmaddr = NXSwapLongLong(sg->vmaddr);
+	sg->vmsize = NXSwapLongLong(sg->vmsize);
+	sg->fileoff = NXSwapLongLong(sg->fileoff);
+	sg->filesize = NXSwapLongLong(sg->filesize);
+	sg->maxprot = NXSwapLong(sg->maxprot);
+	sg->initprot = NXSwapLong(sg->initprot);
+	sg->nsects = NXSwapLong(sg->nsects);
+	sg->flags = NXSwapLong(sg->flags);
+}
+
 void
 swap_section(
 struct section *s,
-unsigned long nsects,
+uint32_t nsects,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
 
 	for(i = 0; i < nsects; i++){
 	    /* sectname[16] */
@@ -113,6 +148,30 @@ enum NXByteOrder target_byte_sex)
 	}
 }
 
+void
+swap_section_64(
+struct section_64 *s,
+uint32_t nsects,
+enum NXByteOrder target_byte_sex)
+{
+    uint32_t i;
+
+	for(i = 0; i < nsects; i++){
+	    /* sectname[16] */
+	    /* segname[16] */
+	    s[i].addr = NXSwapLongLong(s[i].addr);
+	    s[i].size = NXSwapLongLong(s[i].size);
+	    s[i].offset = NXSwapLong(s[i].offset);
+	    s[i].align = NXSwapLong(s[i].align);
+	    s[i].reloff = NXSwapLong(s[i].reloff);
+	    s[i].nreloc = NXSwapLong(s[i].nreloc);
+	    s[i].flags = NXSwapLong(s[i].flags);
+	    s[i].reserved1 = NXSwapLong(s[i].reserved1);
+	    s[i].reserved2 = NXSwapLong(s[i].reserved2);
+	    s[i].reserved3 = NXSwapLong(s[i].reserved3);
+	}
+}
+
 void
 swap_symtab_command(
 struct symtab_command *st,
@@ -301,6 +360,23 @@ enum NXByteOrder target_byte_sex)
 	r_cmd->reserved6 = NXSwapLong(r_cmd->reserved6);
 }
 
+void
+swap_routines_command_64(
+struct routines_command_64 *r_cmd,
+enum NXByteOrder target_byte_sex)
+{
+	r_cmd->cmd = NXSwapLong(r_cmd->cmd);
+	r_cmd->cmdsize = NXSwapLong(r_cmd->cmdsize);
+	r_cmd->init_address = NXSwapLongLong(r_cmd->init_address);
+	r_cmd->init_module = NXSwapLongLong(r_cmd->init_module);
+	r_cmd->reserved1 = NXSwapLongLong(r_cmd->reserved1);
+	r_cmd->reserved2 = NXSwapLongLong(r_cmd->reserved2);
+	r_cmd->reserved3 = NXSwapLongLong(r_cmd->reserved3);
+	r_cmd->reserved4 = NXSwapLongLong(r_cmd->reserved4);
+	r_cmd->reserved5 = NXSwapLongLong(r_cmd->reserved5);
+	r_cmd->reserved6 = NXSwapLongLong(r_cmd->reserved6);
+}
+
 void
 swap_prebind_cksum_command(
 struct prebind_cksum_command *cksum_cmd,
@@ -312,12 +388,21 @@ enum NXByteOrder target_byte_sex)
 }
 
 void
-swap_nlist(
-struct nlist *symbols,
-unsigned long nsymbols,
+swap_uuid_command(
+struct uuid_command *uuid_cmd,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+	uuid_cmd->cmd = NXSwapLong(uuid_cmd->cmd);
+	uuid_cmd->cmdsize = NXSwapLong(uuid_cmd->cmdsize);
+}
+
+void
+swap_nlist(
+struct nlist *symbols,
+uint32_t nsymbols,
+enum NXByteOrder target_byte_sex)
+{
+    uint32_t i;
 
 	for(i = 0; i < nsymbols; i++){
 	    symbols[i].n_un.n_strx = NXSwapLong(symbols[i].n_un.n_strx);
@@ -329,12 +414,29 @@ enum NXByteOrder target_byte_sex)
 }
 
 void
-swap_ranlib(
-struct ranlib *ranlibs,
-unsigned long nranlibs,
+swap_nlist_64(
+struct nlist_64 *symbols,
+uint32_t nsymbols,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
+
+	for(i = 0; i < nsymbols; i++){
+	    symbols[i].n_un.n_strx = NXSwapLong(symbols[i].n_un.n_strx);
+	    /* n_type */
+	    /* n_sect */
+	    symbols[i].n_desc = NXSwapShort(symbols[i].n_desc);
+	    symbols[i].n_value = NXSwapLongLong(symbols[i].n_value);
+	}
+}
+
+void
+swap_ranlib(
+struct ranlib *ranlibs,
+uint32_t nranlibs,
+enum NXByteOrder target_byte_sex)
+{
+    uint32_t i;
 
 	for(i = 0; i < nranlibs; i++){
 	    ranlibs[i].ran_un.ran_strx = NXSwapLong(ranlibs[i].ran_un.ran_strx);
@@ -345,12 +447,12 @@ enum NXByteOrder target_byte_sex)
 void
 swap_relocation_info(
 struct relocation_info *relocs,
-unsigned long nrelocs,
+uint32_t nrelocs,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
     enum NXByteOrder host_byte_sex;
-    unsigned long to_host_byte_sex, scattered;
+    uint32_t to_host_byte_sex, scattered;
 
     struct swapped_relocation_info {
 	long	r_address;
@@ -363,12 +465,12 @@ enum NXByteOrder target_byte_sex)
 		    r_pcrel:1,
 		    r_symbolnum:24;
 	    } fields;
-	    unsigned long word;
+	    uint32_t word;
 	} u;
     } sr;
 
     struct swapped_scattered_relocation_info {
-	unsigned long word;
+	uint32_t word;
 	long	r_value;
     } *ssr;
 
@@ -414,11 +516,11 @@ enum NXByteOrder target_byte_sex)
 
 void
 swap_indirect_symbols(
-unsigned long *indirect_symbols,
-unsigned long nindirect_symbols,
+uint32_t *indirect_symbols,
+uint32_t nindirect_symbols,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
 
 	for(i = 0; i < nindirect_symbols; i++)
 	    indirect_symbols[i] = NXSwapLong(indirect_symbols[i]);
@@ -427,21 +529,21 @@ enum NXByteOrder target_byte_sex)
 void
 swap_dylib_reference(
 struct dylib_reference *refs,
-unsigned long nrefs,
+uint32_t nrefs,
 enum NXByteOrder target_byte_sex)
 {
     struct swapped_dylib_reference {
 	union {
 	    struct {
-		unsigned long
+		uint32_t
 		    flags:8,
 		    isym:24;
 	    } fields;
-	    unsigned long word;
+	    uint32_t word;
 	} u;
     } sref;
 
-    unsigned long i;
+    uint32_t i;
     enum NXByteOrder host_byte_sex;
 
 	host_byte_sex = NXHostByteOrder();
@@ -466,10 +568,10 @@ enum NXByteOrder target_byte_sex)
 void
 swap_dylib_module(
 struct dylib_module *mods,
-unsigned long nmods,
+uint32_t nmods,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
 
 	for(i = 0; i < nmods; i++){
 	    mods[i].module_name = NXSwapLong(mods[i].module_name);
@@ -483,19 +585,47 @@ enum NXByteOrder target_byte_sex)
 	    mods[i].nextrel     = NXSwapLong(mods[i].nextrel);
 	    mods[i].iinit_iterm = NXSwapLong(mods[i].iinit_iterm);
 	    mods[i].ninit_nterm = NXSwapLong(mods[i].ninit_nterm);
+	    mods[i].objc_module_info_size = NXSwapLong(mods[i].objc_module_info_size);
+	    mods[i].objc_module_info_addr = NXSwapLong(mods[i].objc_module_info_addr);
+	}
+}
+
+void
+swap_dylib_module_64(
+struct dylib_module_64 *mods,
+uint32_t nmods,
+enum NXByteOrder target_byte_sex)
+{
+    uint32_t i;
+
+	for(i = 0; i < nmods; i++){
+	    mods[i].module_name = NXSwapLong(mods[i].module_name);
+	    mods[i].iextdefsym  = NXSwapLong(mods[i].iextdefsym);
+	    mods[i].nextdefsym  = NXSwapLong(mods[i].nextdefsym);
+	    mods[i].irefsym     = NXSwapLong(mods[i].irefsym);
+	    mods[i].nrefsym     = NXSwapLong(mods[i].nrefsym);
+	    mods[i].ilocalsym   = NXSwapLong(mods[i].ilocalsym);
+	    mods[i].nlocalsym   = NXSwapLong(mods[i].nlocalsym);
+	    mods[i].iextrel     = NXSwapLong(mods[i].iextrel);
+	    mods[i].nextrel     = NXSwapLong(mods[i].nextrel);
+	    mods[i].iinit_iterm = NXSwapLong(mods[i].iinit_iterm);
+	    mods[i].ninit_nterm = NXSwapLong(mods[i].ninit_nterm);
+	    mods[i].objc_module_info_size = NXSwapLong(mods[i].objc_module_info_size);
+	    mods[i].objc_module_info_addr = NXSwapLongLong(mods[i].objc_module_info_addr);
 	}
 }
 
 void
 swap_dylib_table_of_contents(
 struct dylib_table_of_contents *tocs,
-unsigned long ntocs,
+uint32_t ntocs,
 enum NXByteOrder target_byte_sex)
 {
-    unsigned long i;
+    uint32_t i;
 
 	for(i = 0; i < ntocs; i++){
 	    tocs[i].symbol_index = NXSwapLong(tocs[i].symbol_index);
 	    tocs[i].module_index = NXSwapLong(tocs[i].module_index);
 	}
 }
+#endif /* !defined(RLD) */
