@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -30,6 +29,7 @@
  *   -info
  *   -detailed_info
  *   -arch <arch_type> <input_file>
+ *   -arch_blank <arch_type>
  *   -thin <arch_type>
  *   -extract <arch_type>
  *   -remove <arch_type>
@@ -104,7 +104,7 @@ static time_t output_timep[2] = { 0 };
 #endif
 static enum bool archives_in_input = FALSE;
 
-/* flags set from command line arguments to specifify the operation */
+/* flags set from command line arguments to specify the operation */
 static enum bool create_flag = FALSE;
 static enum bool info_flag = FALSE;
 static enum bool detailed_info_flag = FALSE;
@@ -136,6 +136,7 @@ struct segalign {
 static struct segalign *segaligns = NULL;
 static unsigned long nsegaligns = 0;
 
+static enum bool arch_blank_flag = FALSE;
 
 static struct fat_header fat_header = { 0 };
 
@@ -162,6 +163,12 @@ static unsigned long get_align(
     unsigned long size,
     char *name,
     enum bool swapped);
+static unsigned long get_align_64(
+    struct mach_header_64 *mhp64,
+    struct load_command *load_commands,
+    unsigned long size,
+    char *name,
+    enum bool swapped);
 static unsigned long guess_align(
     unsigned long vmaddr);
 static void print_arch(
@@ -195,6 +202,8 @@ static void check_arch(
     struct thin_file *thin);
 static void usage(
     void);
+static struct thin_file* new_blank_dylib(
+    struct arch_flag arch);
 
 int
 main(
@@ -202,7 +211,7 @@ int argc,
 char *argv[],
 char *envp[])
 {
-    int fd;
+    int fd, a;
     unsigned long i, j, k, value;
     char *p, *endp;
     struct input_file *input;
@@ -211,33 +220,50 @@ char *envp[])
     struct segalign *segalign;
     const struct arch_flag *arch_flags;
     enum bool found;
+    struct arch_flag blank_arch;
 
 	input = NULL;
 	/*
 	 * Process the command line arguments.
 	 */
 	progname = argv[0];
-	for(i = 1; i < argc; i++){
-	    if(argv[i][0] == '-'){
-		p = &(argv[i][1]);
+	for(a = 1; a < argc; a++){
+	    if(argv[a][0] == '-'){
+		p = &(argv[a][1]);
 		switch(*p){
 		case 'a':
 		    if(strcmp(p, "arch") == 0 || strcmp(p, "a") == 0){
-			if(i + 2 >= argc){
-			    error("missing argument(s) to %s option", argv[i]);
+			if(a + 2 >= argc){
+			    error("missing argument(s) to %s option", argv[a]);
 			    usage();
 			}
 			input = new_input();
-			if(get_arch_from_flag(argv[i+1],
+			if(get_arch_from_flag(argv[a+1],
 					      &(input->arch_flag)) == 0){
 			    error("unknown architecture specification flag: %s "
-				  "in specifying input file %s %s %s", argv[i+1],
-				  argv[i], argv[i+1], argv[i+2]);
+				  "in specifying input file %s %s %s", argv[a+1],
+				  argv[a], argv[a+1], argv[a+2]);
 			    arch_usage();
 			    usage();
 			}
-			input->name = argv[i+2];
-			i += 2;
+			input->name = argv[a+2];
+			a += 2;
+		    }
+		    else if(strcmp(p, "arch_blank") == 0){
+		    arch_blank_flag = TRUE;
+			if(a + 1 >= argc){
+			    error("missing argument(s) to %s option", argv[a]);
+			    usage();
+			}
+			if(get_arch_from_flag(argv[a+1], &blank_arch) == 0){
+			    error("unknown architecture specification flag: %s "
+				  "in specifying input file %s %s", argv[a+1],
+				  argv[a], argv[a+1]);
+			    arch_usage();
+			    usage();
+			}
+			new_blank_dylib(blank_arch);
+			a += 1;
 		    }
 		    else
 			goto unknown_flag;
@@ -263,20 +289,20 @@ char *envp[])
 			extract_flag = TRUE;
 			if(strcmp(p, "extract_family") == 0)
 			    extract_family_flag = TRUE;
-			if(i + 1 >= argc){
-			    error("missing argument to %s option", argv[i]);
+			if(a + 1 >= argc){
+			    error("missing argument to %s option", argv[a]);
 			    usage();
 			}
 			arch_flag = new_arch_flag(&extract_arch_flags,
 						&nextract_arch_flags);
-			if(get_arch_from_flag(argv[i+1], arch_flag) == 0){
+			if(get_arch_from_flag(argv[a+1], arch_flag) == 0){
 			    error("unknown architecture specification flag: "
 				  "%s in specifying extract operation: %s %s",
-				  argv[i+1], argv[i], argv[i+1]);
+				  argv[a+1], argv[a], argv[a+1]);
 			    arch_usage();
 			    usage();
 			}
-			i++;
+			a++;
 		    }
 		    else
 			goto unknown_flag;
@@ -290,14 +316,14 @@ char *envp[])
 		    break;
 		case 'o':
 		    if(strcmp(p, "output") == 0 || strcmp(p, "o") == 0){
-			if(i + 1 >= argc){
-			    error("missing argument to %s option", argv[i]);
+			if(a + 1 >= argc){
+			    error("missing argument to %s option", argv[a]);
 			    usage();
 			}
 			if(output_file != NULL)
-			    fatal("more than one %s option specified", argv[i]);
-			output_file = argv[i + 1];
-			i++;
+			    fatal("more than one %s option specified", argv[a]);
+			output_file = argv[a + 1];
+			a++;
 		    }
 		    else
 			goto unknown_flag;
@@ -305,61 +331,61 @@ char *envp[])
 		case 'r':
 		    if(strcmp(p, "remove") == 0 || strcmp(p, "rem") == 0){
 			remove_flag = TRUE;
-			if(i + 1 >= argc){
-			    error("missing argument to %s option", argv[i]);
+			if(a + 1 >= argc){
+			    error("missing argument to %s option", argv[a]);
 			    usage();
 			}
 			arch_flag = new_arch_flag(&remove_arch_flags,
 						&nremove_arch_flags);
-			if(get_arch_from_flag(argv[i+1], arch_flag) == 0){
+			if(get_arch_from_flag(argv[a+1], arch_flag) == 0){
 			    error("unknown architecture specification flag: "
 				  "%s in specifying remove operation: %s %s",
-				  argv[i+1], argv[i], argv[i+1]);
+				  argv[a+1], argv[a], argv[a+1]);
 			    arch_usage();
 			    usage();
 			}
-			i++;
+			a++;
 		    }
 		    else if(strcmp(p, "replace") == 0 || strcmp(p, "rep") == 0){
 			replace_flag = TRUE;
-			if(i + 2 >= argc){
-			    error("missing argument(s) to %s option", argv[i]);
+			if(a + 2 >= argc){
+			    error("missing argument(s) to %s option", argv[a]);
 			    usage();
 			}
 			replace = new_replace();
-			if(get_arch_from_flag(argv[i+1],
+			if(get_arch_from_flag(argv[a+1],
 					      &(replace->arch_flag)) == 0){
 			    error("unknown architecture specification flag: "
 				  "%s in specifying replace operation: %s %s %s",
-				  argv[i+1], argv[i], argv[i+1], argv[i+2]);
+				  argv[a+1], argv[a], argv[a+1], argv[a+2]);
 			    arch_usage();
 			    usage();
 			}
-			replace->thin_file.name = argv[i+2];
-			i += 2;
+			replace->thin_file.name = argv[a+2];
+			a += 2;
 		    }
 		    else
 			goto unknown_flag;
 		    break;
 		case 's':
 		    if(strcmp(p, "segalign") == 0 || strcmp(p, "s") == 0){
-			if(i + 2 >= argc){
-			    error("missing argument(s) to %s option", argv[i]);
+			if(a + 2 >= argc){
+			    error("missing argument(s) to %s option", argv[a]);
 			    usage();
 			}
 			segalign = new_segalign();
-			if(get_arch_from_flag(argv[i+1],
+			if(get_arch_from_flag(argv[a+1],
 					      &(segalign->arch_flag)) == 0){
 			    error("unknown architecture specification flag: "
 				  "%s in specifying segment alignment: %s %s %s",
-				  argv[i+1], argv[i], argv[i+1], argv[i+2]);
+				  argv[a+1], argv[a], argv[a+1], argv[a+2]);
 			    arch_usage();
 			    usage();
 			}
-			value = strtoul(argv[i+2], &endp, 16);
+			value = strtoul(argv[a+2], &endp, 16);
 			if(*endp != '\0')
 			    fatal("argument for -segalign <arch_type> %s not a "
-				  "proper hexadecimal number", argv[i+2]);
+				  "proper hexadecimal number", argv[a+2]);
 			if(!ispoweroftwo(value) || value == 0)
 			    fatal("argument to -segalign <arch_type> %lx (hex) "
 				  "must be a non-zero power of two", value);
@@ -372,7 +398,7 @@ char *envp[])
 			    value >>= 1;
 			    segalign->align++;
 			}
-			i += 2;
+			a += 2;
 		    }
 		    else
 			goto unknown_flag;
@@ -380,32 +406,32 @@ char *envp[])
 		case 't':
 		    if(strcmp(p, "thin") == 0 || strcmp(p, "t") == 0){
 			if(thin_flag == TRUE)
-			    fatal("more than one %s option specified", argv[i]);
+			    fatal("more than one %s option specified", argv[a]);
 			thin_flag = TRUE;
-			if(i + 1 >= argc){
-			    error("missing argument to %s option", argv[i]);
+			if(a + 1 >= argc){
+			    error("missing argument to %s option", argv[a]);
 			    usage();
 			}
-			if(get_arch_from_flag(argv[i+1], &thin_arch_flag) == 0){
+			if(get_arch_from_flag(argv[a+1], &thin_arch_flag) == 0){
 			    error("unknown architecture specification flag: "
 				  "%s in specifying thin operation: %s %s",
-				  argv[i+1], argv[i], argv[i+1]);
+				  argv[a+1], argv[a], argv[a+1]);
 			    arch_usage();
 			    usage();
 			}
-			i++;
+			a++;
 		    }
 		    else
 			goto unknown_flag;
 		    break;
 		default:
 unknown_flag:
-		    fatal("unknown flag: %s", argv[i]);
+		    fatal("unknown flag: %s", argv[a]);
 		}
 	    }
 	    else{
 		input = new_input();
-		input->name = argv[i];
+		input->name = argv[a];
 	    }
 	}
 
@@ -438,6 +464,10 @@ unknown_flag:
 		  "-info or -detailed_info can be specified");
 	    usage();
 	}
+	if(arch_blank_flag == TRUE && create_flag == FALSE){
+		error("-arch_blank may only be used with -create");
+		usage();
+	}
 
 	/*
 	 * Determine the types of the input files.
@@ -495,7 +525,7 @@ unknown_flag:
 				     output_file);
 
 		    if(write(fd, thin_files[i].addr,thin_files[i].fat_arch.size)
-		       != thin_files[i].fat_arch.size)
+		       != (int)(thin_files[i].fat_arch.size))
 			system_fatal("can't write thin file to output file: %s",
 				     output_file);
 		    if(close(fd) == -1)
@@ -682,7 +712,7 @@ unknown_flag:
 		    printf("Fat header in: %s\n", input_files[i].name);
 		    printf("fat_magic 0x%x\n",
 			  (unsigned int)(input_files[i].fat_header->magic));
-		    printf("nfat_arch %lu\n",
+		    printf("nfat_arch %u\n",
 			   input_files[i].fat_header->nfat_arch);
 		    for(j = 0; j < input_files[i].fat_header->nfat_arch; j++){
 			printf("architecture ");
@@ -690,11 +720,11 @@ unknown_flag:
 			printf("\n");
 			print_cputype(input_files[i].fat_arches[j].cputype,
 				      input_files[i].fat_arches[j].cpusubtype);
-			printf("    offset %lu\n",
+			printf("    offset %u\n",
 			       input_files[i].fat_arches[j].offset);
-			printf("    size %lu\n",
+			printf("    size %u\n",
 			       input_files[i].fat_arches[j].size);
-			printf("    align 2^%lu (%d)\n",
+			printf("    align 2^%u (%d)\n",
 			       input_files[i].fat_arches[j].align,
 			       1 << input_files[i].fat_arches[j].align);
 		    }
@@ -826,21 +856,11 @@ create_fat(void)
 		if(lseek(fd, thin_files[i].fat_arch.offset, L_SET) == -1)
 		    system_fatal("can't lseek in output file: %s", output_file);
 	    if(write(fd, thin_files[i].addr, thin_files[i].fat_arch.size)
-	       != thin_files[i].fat_arch.size)
+	       != (int)(thin_files[i].fat_arch.size))
 		system_fatal("can't write to output file: %s", output_file);
 	}
 	if(close(fd) == -1)
 	    system_fatal("can't close output file: %s", output_file);
-	if(archives_in_input == TRUE){
-	    if(utime(output_file,
-#ifndef __OPENSTEP__
-		     &output_timep) == -1)
-#else
-		     output_timep) == -1)
-#endif
-		system_fatal("can't set the modifiy times for output file: %s",
-			     output_file);
-	}
 }
 
 /*
@@ -859,6 +879,7 @@ struct input_file *input)
     char *addr;
     struct thin_file *thin;
     struct mach_header *mhp, mh;
+    struct mach_header_64 *mhp64, mh64;
     struct load_command *lcp;
     cpu_type_t cputype;
     cpu_subtype_t cpusubtype;
@@ -935,15 +956,15 @@ struct input_file *input)
 			  "end of the file) %s", input->fat_arches[i].cputype,
 			  input->fat_arches[i].cpusubtype, input->name);
 		if(input->fat_arches[i].align > MAXSECTALIGN)
-		    fatal("align (2^%lu) too large of fat file %s (cputype (%d)"
+		    fatal("align (2^%u) too large of fat file %s (cputype (%d)"
 			  " cpusubtype (%d)) (maximum 2^%d)",
 			  input->fat_arches[i].align, input->name,
 			  input->fat_arches[i].cputype,
 			  input->fat_arches[i].cpusubtype, MAXSECTALIGN);
 		if(input->fat_arches[i].offset %
 		   (1 << input->fat_arches[i].align) != 0)
-		    fatal("offset %lu of fat file %s (cputype (%d) cpusubtype "
-			  "(%d)) not aligned on it's alignment (2^%lu)",
+		    fatal("offset %u of fat file %s (cputype (%d) cpusubtype "
+			  "(%d)) not aligned on it's alignment (2^%u)",
 			  input->fat_arches[i].offset, input->name,
 			  input->fat_arches[i].cputype,
 			  input->fat_arches[i].cpusubtype,
@@ -974,7 +995,7 @@ struct input_file *input)
 		    archives_in_input = TRUE;
 	    }
 	}
-	/* see if this file is Mach-O file */
+	/* see if this file is Mach-O file for 32-bit architectures */
 	else if(size >= sizeof(struct mach_header) &&
 	        (*((unsigned long *)addr) == MH_MAGIC ||
 	         *((unsigned long *)addr) == SWAP_LONG(MH_MAGIC))){
@@ -1006,6 +1027,38 @@ struct input_file *input)
 	    if(input->arch_flag.name != NULL)
 		check_arch(input, thin);
 	}
+	/* see if this file is Mach-O file for 64-bit architectures */
+	else if(size >= sizeof(struct mach_header_64) &&
+	        (*((unsigned long *)addr) == MH_MAGIC_64 ||
+	         *((unsigned long *)addr) == SWAP_LONG(MH_MAGIC_64))){
+
+	    /* this is a Mach-O file so create a thin file struct for it */
+	    thin = new_thin();
+	    input->is_thin = TRUE;
+	    thin->name = input->name;
+	    thin->addr = addr;
+	    mhp64 = (struct mach_header_64 *)addr;
+	    lcp = (struct load_command *)((char *)mhp64 +
+					  sizeof(struct mach_header_64));
+	    if(mhp64->magic == SWAP_LONG(MH_MAGIC_64)){
+		swapped = TRUE;
+		mh64 = *mhp64;
+		swap_mach_header_64(&mh64, get_host_byte_sex());
+		mhp64 = &mh64;
+	    }
+	    else
+		swapped = FALSE;
+	    thin->fat_arch.cputype = mhp64->cputype;
+	    thin->fat_arch.cpusubtype = mhp64->cpusubtype;
+	    thin->fat_arch.offset = 0;
+	    thin->fat_arch.size = size;
+	    thin->fat_arch.align = get_align_64(mhp64, lcp, size, input->name,
+					        swapped);
+
+	    /* if the arch type is specified make sure it matches the object */
+	    if(input->arch_flag.name != NULL)
+		check_arch(input, thin);
+	}
 	/* see if this file is an archive file */
 	else if(size >= SARMAG && strncmp(addr, ARMAG, SARMAG) == 0){
 
@@ -1073,6 +1126,7 @@ struct replace *replace)
     kern_return_t r;
     char *addr;
     struct mach_header *mhp, mh;
+    struct mach_header_64 *mhp64, mh64;
     struct load_command *lcp;
     cpu_type_t cputype;
     cpu_subtype_t cpusubtype;
@@ -1101,7 +1155,7 @@ struct replace *replace)
 	    fatal("replacement file: %s is a fat file (must be a thin file)",
 		  replace->thin_file.name);
 	}
-	/* see if this file is Mach-O file */
+	/* see if this file is Mach-O file for 32-bit architectures */
 	else if(size >= sizeof(struct mach_header) &&
 	        (*((unsigned long *)addr) == MH_MAGIC ||
 	         *((unsigned long *)addr) == SWAP_LONG(MH_MAGIC))){
@@ -1126,6 +1180,31 @@ struct replace *replace)
 	    replace->thin_file.fat_arch.align =
 		    get_align(mhp, lcp, size, replace->thin_file.name, swapped);
 	}
+	/* see if this file is Mach-O file for 64-bit architectures */
+	else if(size >= sizeof(struct mach_header_64) &&
+	        (*((unsigned long *)addr) == MH_MAGIC_64 ||
+	         *((unsigned long *)addr) == SWAP_LONG(MH_MAGIC_64))){
+
+	    /* this is a Mach-O file so fill in the thin file struct for it */
+	    replace->thin_file.addr = addr;
+	    mhp64 = (struct mach_header_64 *)addr;
+	    lcp = (struct load_command *)((char *)mhp64 +
+					  sizeof(struct mach_header_64));
+	    if(mhp64->magic == SWAP_LONG(MH_MAGIC_64)){
+		swapped = TRUE;
+		mh64 = *mhp64;
+		swap_mach_header_64(&mh64, get_host_byte_sex());
+		mhp64 = &mh64;
+	    }
+	    else
+		swapped = FALSE;
+	    replace->thin_file.fat_arch.cputype = mhp64->cputype;
+	    replace->thin_file.fat_arch.cpusubtype = mhp64->cpusubtype;
+	    replace->thin_file.fat_arch.offset = 0;
+	    replace->thin_file.fat_arch.size = size;
+	    replace->thin_file.fat_arch.align =
+	       get_align_64(mhp64, lcp, size, replace->thin_file.name, swapped);
+	}
 	/* see if this file is an archive file */
 	else if(size >= SARMAG && strncmp(addr, ARMAG, SARMAG) == 0){
 
@@ -1175,6 +1254,7 @@ cpu_subtype_t *cpusubtype)
 {
     unsigned long offset, magic, i, ar_name_size;
     struct mach_header mh;
+    struct mach_header_64 mh64;
     struct ar_hdr *ar_hdr;
     char *ar_name;
 
@@ -1231,6 +1311,26 @@ cpu_subtype_t *cpusubtype)
 			      *cputype, *cpusubtype);
 		    }
 		}
+		else if((size - ar_name_size) - offset >=
+		    sizeof(struct mach_header_64) &&
+		   (magic == MH_MAGIC_64 || magic == SWAP_LONG(MH_MAGIC_64))){
+		    memcpy(&mh64, addr + offset + ar_name_size,
+			   sizeof(struct mach_header_64));
+		    if(mh64.magic == SWAP_LONG(MH_MAGIC_64))
+			swap_mach_header_64(&mh64, get_host_byte_sex());
+		    if(*cputype == 0){
+			*cputype = mh64.cputype;
+			*cpusubtype = mh64.cpusubtype;
+		    }
+		    else if(*cputype != mh64.cputype){
+			fatal("archive member %s(%.*s) cputype (%d) and "
+			      "cpusubtype (%d) does not match previous "
+			      "archive members cputype (%d) and cpusubtype"
+			      " (%d) (all members must match)", name,
+			      (int)i, ar_name, mh64.cputype, mh64.cpusubtype,
+			      *cputype, *cpusubtype);
+		    }
+		}
 	    }
 	    offset += round(strtoul(ar_hdr->ar_size, NULL, 10),
 			    sizeof(short));
@@ -1299,6 +1399,17 @@ enum bool swapped)
     struct section *sp, s;
     enum byte_sex host_byte_sex;
 
+	/*
+	 * Special case ppc, ppc64, i386 and x86_64 architectures and return 12.
+	 * We know that with those architectures that the kernel and mmap only
+	 * need file offsets to be page (4096 byte) aligned.
+	 */
+	if(mhp->cputype == CPU_TYPE_POWERPC ||
+	   mhp->cputype == CPU_TYPE_POWERPC64 ||
+	   mhp->cputype == CPU_TYPE_I386 ||
+	   mhp->cputype == CPU_TYPE_X86_64)
+	    return(12);
+
 	host_byte_sex = get_host_byte_sex();
 
 	/* set worst case the link editor uses first */
@@ -1354,6 +1465,95 @@ enum bool swapped)
 	return(cur_align);
 }
 
+/*
+ * get_align_64 is passed a pointer to a mach_header_64 and size of the object.
+ * It returns the segment alignment the object was created with.  It guesses but
+ * it is conservative.  The maximum alignment is that the link editor will allow
+ * MAXSECTALIGN and the minimum is the conserative alignment for a long long
+ * which appears in a mach object files (2^3 worst case for all 64-bit
+ * machines).
+ */
+static
+unsigned long
+get_align_64(
+struct mach_header_64 *mhp64,
+struct load_command *load_commands,
+unsigned long size,
+char *name,
+enum bool swapped)
+{
+    unsigned long i, j, cur_align, align;
+    struct load_command *lcp, l;
+    struct segment_command_64 *sgp, sg;
+    struct section_64 *sp, s;
+    enum byte_sex host_byte_sex;
+
+	/*
+	 * Special case ppc, ppc64, i386 and x86_64 architectures and return 12.
+	 * We know that with those architectures that the kernel and mmap only
+	 * need file offsets to be page (4096 byte) aligned.
+	 */
+	if(mhp64->cputype == CPU_TYPE_POWERPC ||
+	   mhp64->cputype == CPU_TYPE_POWERPC64 ||
+	   mhp64->cputype == CPU_TYPE_I386 ||
+	   mhp64->cputype == CPU_TYPE_X86_64)
+	    return(12);
+
+	host_byte_sex = get_host_byte_sex();
+
+	/* set worst case the link editor uses first */
+	cur_align = MAXSECTALIGN;
+	if(mhp64->sizeofcmds + sizeof(struct mach_header_64) > size)
+	    fatal("truncated or malformed object (load commands would "
+		  "extend past the end of the file) in: %s", name);
+	lcp = load_commands;
+	for(i = 0; i < mhp64->ncmds; i++){
+	    l = *lcp;
+	    if(swapped)
+		swap_load_command(&l, host_byte_sex);
+	    if(l.cmdsize % sizeof(long long) != 0)
+		error("load command %lu size not a multiple of "
+		      "sizeof(long long) in: %s", i, name);
+	    if(l.cmdsize <= 0)
+		fatal("load command %lu size is less than or equal to zero "
+		      "in: %s", i, name);
+	    if((char *)lcp + l.cmdsize >
+	       (char *)load_commands + mhp64->sizeofcmds)
+		fatal("load command %lu extends past end of all load "
+		      "commands in: %s", i, name);
+	    if(l.cmd == LC_SEGMENT_64){
+		sgp = (struct segment_command_64 *)lcp;
+		sg = *sgp;
+		if(swapped)
+		    swap_segment_command_64(&sg, host_byte_sex);
+		if(mhp64->filetype == MH_OBJECT){
+		    /* this is the minimum alignment, then take largest */
+		    align = 3; /* 2^3 sizeof(long long) */
+		    sp = (struct section_64 *)((char *)sgp +
+					    sizeof(struct segment_command_64));
+		    for(j = 0; j < sg.nsects; j++){
+			s = *sp;
+			if(swapped)
+			    swap_section_64(&s, 1, host_byte_sex);
+			if(s.align > align)
+			    align = s.align;
+			sp++;
+		    }
+		    if(align < cur_align)
+			cur_align = align;
+		}
+		else{
+		    /* guess the smallest alignment and use that */
+		    align = guess_align(sg.vmaddr);
+		    if(align < cur_align)
+			cur_align = align;
+		}
+	    }
+	    lcp = (struct load_command *)((char *)lcp + l.cmdsize);
+	}
+	return(cur_align);
+}
+
 /*
  * guess_align is passed a vmaddr of a segment and guesses what the segment
  * alignment was.  It uses the most conservative guess up to the maximum
@@ -1441,6 +1641,21 @@ struct fat_arch *fat_arch)
 	    case CPU_SUBTYPE_POWERPC_7450:
 		printf("ppc7450");
 		break;
+	    case CPU_SUBTYPE_POWERPC_970:
+		printf("ppc970");
+		break;
+	    default:
+		goto print_arch_unknown;
+	    }
+	    break;
+	case CPU_TYPE_POWERPC64:
+	    switch(fat_arch->cpusubtype){
+	    case CPU_SUBTYPE_POWERPC_ALL:
+		printf("ppc64");
+		break;
+	    case CPU_SUBTYPE_POWERPC_970:
+		printf("ppc970-64");
+		break;
 	    default:
 		goto print_arch_unknown;
 	    }
@@ -1495,6 +1710,15 @@ struct fat_arch *fat_arch)
 		goto print_arch_unknown;
 	    }
 	    break;
+	case CPU_TYPE_X86_64:
+	    switch(fat_arch->cpusubtype){
+	    case CPU_SUBTYPE_X86_64_ALL:
+		printf("x86_64");
+		break;
+	    default:
+		goto print_arch_unknown;
+	    }
+	    break;
 	case CPU_TYPE_I860:
 	    switch(fat_arch->cpusubtype){
 	    case CPU_SUBTYPE_I860_ALL:
@@ -1618,6 +1842,24 @@ cpu_subtype_t cpusubtype)
 		printf("    cputype CPU_TYPE_POWERPC\n"
 		       "    cpusubtype CPU_SUBTYPE_POWERPC_7450\n");
 		break;
+	    case CPU_SUBTYPE_POWERPC_970:
+		printf("    cputype CPU_TYPE_POWERPC\n"
+		       "    cpusubtype CPU_SUBTYPE_POWERPC_970\n");
+		break;
+	    default:
+		goto print_arch_unknown;
+	    }
+	    break;
+	case CPU_TYPE_POWERPC64:
+	    switch(cpusubtype){
+	    case CPU_SUBTYPE_POWERPC_ALL:
+		printf("    cputype CPU_TYPE_POWERPC64\n"
+		       "    cpusubtype CPU_SUBTYPE_POWERPC_ALL\n");
+		break;
+	    case CPU_SUBTYPE_POWERPC_970:
+		printf("    cputype CPU_TYPE_POWERPC64\n"
+		       "    cpusubtype CPU_SUBTYPE_POWERPC_970\n");
+		break;
 	    default:
 		goto print_arch_unknown;
 	    }
@@ -1685,6 +1927,16 @@ cpu_subtype_t cpusubtype)
 		goto print_arch_unknown;
 	    }
 	    break;
+	case CPU_TYPE_X86_64:
+	    switch(cpusubtype){
+	    case CPU_SUBTYPE_X86_64_ALL:
+		printf("    cputype CPU_TYPE_X86_64\n"
+		       "    cpusubtype CPU_SUBTYPE_X86_64_ALL\n");
+		break;
+	    default:
+		goto print_arch_unknown;
+	    }
+	    break;
 	case CPU_TYPE_I860:
 	    switch(cpusubtype){
 	    case CPU_SUBTYPE_I860_ALL:
@@ -1758,7 +2010,7 @@ int
 size_ar_name(
 char *ar_name)
 {
-    int j;
+    unsigned long j;
     struct ar_hdr ar_hdr;
 
 	for(j = 0; j < sizeof(ar_hdr.ar_name); j++){
@@ -1922,6 +2174,80 @@ struct thin_file *thin)
 		  input->arch_flag.cputype, input->arch_flag.cpusubtype);
 }
 
+/*
+ * Create a blank dylib.  This is a stub dylib with no load commands.
+ * It is 4096 bytes of zero except for the mach_header.
+ */
+static
+struct thin_file*
+new_blank_dylib(struct arch_flag arch)
+{
+	struct thin_file *file = new_thin();
+	struct mach_header *mh;
+	struct mach_header_64 *mh64;
+	file->name = "blank dylib";
+	file->addr = calloc(1, 4096);
+	file->fat_arch.cputype = 0;
+	file->fat_arch.cpusubtype = 0;
+	file->fat_arch.offset = 0;
+	file->fat_arch.size = 4096;
+	file->fat_arch.align = 12;
+    
+	switch(arch.cputype)
+	{
+		case CPU_TYPE_POWERPC:
+			file->fat_arch.cputype = CPU_TYPE_POWERPC;
+			file->fat_arch.cpusubtype = CPU_SUBTYPE_POWERPC_ALL;
+			mh = (struct mach_header *) file->addr;
+			mh->magic = MH_MAGIC;
+			mh->cputype = CPU_TYPE_POWERPC;
+			mh->cpusubtype = CPU_SUBTYPE_POWERPC_ALL;
+			mh->filetype = MH_DYLIB_STUB;
+#if __LITTLE_ENDIAN__
+			swap_mach_header(mh, get_host_byte_sex());
+#endif
+			break;
+		case CPU_TYPE_POWERPC64:
+			file->fat_arch.cputype = CPU_TYPE_POWERPC64;
+			file->fat_arch.cpusubtype = CPU_SUBTYPE_POWERPC_ALL;
+			mh64 = (struct mach_header_64 *) file->addr;
+			mh64->magic = MH_MAGIC_64;
+			mh64->cputype = CPU_TYPE_POWERPC64;
+			mh64->cpusubtype = CPU_SUBTYPE_POWERPC_ALL;
+			mh64->filetype = MH_DYLIB_STUB;
+#if __LITTLE_ENDIAN__
+			swap_mach_header_64(mh64, get_host_byte_sex());
+#endif
+			break;
+		case CPU_TYPE_I386:
+			file->fat_arch.cputype = CPU_TYPE_I386;
+			file->fat_arch.cpusubtype = CPU_SUBTYPE_I386_ALL;
+			mh = (struct mach_header *) file->addr;
+			mh->magic = MH_MAGIC;
+			mh->cputype = CPU_TYPE_I386;
+			mh->cpusubtype = CPU_SUBTYPE_I386_ALL;
+			mh->filetype = MH_DYLIB_STUB;
+#if __BIG_ENDIAN__
+			swap_mach_header(mh, get_host_byte_sex());
+#endif
+			break;
+		case CPU_TYPE_X86_64:
+			file->fat_arch.cputype = CPU_TYPE_X86_64;
+			file->fat_arch.cpusubtype = CPU_SUBTYPE_X86_64_ALL;
+			mh64 = (struct mach_header_64 *) file->addr;
+			mh64->magic = MH_MAGIC_64;
+			mh64->cputype = CPU_TYPE_X86_64;
+			mh64->cpusubtype = CPU_SUBTYPE_X86_64_ALL;
+			mh64->filetype = MH_DYLIB_STUB;
+#if __BIG_ENDIAN__
+			swap_mach_header_64(mh64, get_host_byte_sex());
+#endif
+			break;
+	}
+	
+	return file;
+}
+
 /*
  * Print the current usage line and exit (by calling fatal).
  */
@@ -1931,7 +2257,8 @@ usage(void)
 {
 	fatal("Usage: %s [input_file] ... [-arch <arch_type> input_file] ... "
 	      "[-info] [-detailed_info] [-output output_file] [-create] "
-	      "[-thin <arch_type>] [-remove <arch_type>] ... "
-	      "[-extract <arch_type>] ... [-extract_family <arch_type>] ..."
+	      "[-arch_blank <arch_type>] [-thin <arch_type>] "
+	      "[-remove <arch_type>] ... [-extract <arch_type>] ... "
+	      "[-extract_family <arch_type>] ..."
 	      "[-replace <arch_type> <file_name>] ...", progname);
 }
